@@ -8,7 +8,7 @@ import os
 import sys
 import textwrap
 import warnings
-from typing import Any, Collection, Iterable, cast
+from typing import Collection, Iterable, cast
 
 from resolvelib.reporters import BaseReporter
 from resolvelib.resolvers import ResolutionImpossible, ResolutionTooDeep, Resolver
@@ -26,13 +26,14 @@ from pdm.cli.utils import (
     set_env_in_reg,
 )
 from pdm.environments import BareEnvironment
-from pdm.exceptions import PdmUsageError, ProjectError
+from pdm.exceptions import PdmException, PdmUsageError, ProjectError
 from pdm.models.candidates import Candidate
+from pdm.models.repositories import LockedRepository
 from pdm.models.requirements import Requirement, parse_requirement
 from pdm.project import Project
-from pdm.project.lockfile import FLAG_CROSS_PLATFORM, FLAG_DIRECT_MINIMAL_VERSIONS
+from pdm.project.lockfile import FLAG_CROSS_PLATFORM, FLAG_DIRECT_MINIMAL_VERSIONS, FLAG_INHERIT_METADATA
 from pdm.resolver import resolve
-from pdm.utils import deprecation_warning
+from pdm.termui import logger
 
 
 def do_lock(
@@ -49,6 +50,8 @@ def do_lock(
     """Performs the locking process and update lockfile."""
     hooks = hooks or HookManager(project)
     check_project_file(project)
+    if not project.config["strategy.inherit_metadata"]:
+        project.lockfile.default_strategies.remove(FLAG_INHERIT_METADATA)
     lock_strategy = project.lockfile.apply_strategy_change(strategy_change or [])
     if refresh:
         locked_repo = project.locked_repository
@@ -101,6 +104,7 @@ def do_lock(
                     requirements,
                     project.environment.python_requires,
                     resolve_max_rounds,
+                    inherit_metadata=FLAG_INHERIT_METADATA in lock_strategy,
                 )
                 spin.update("Fetching hashes for resolved packages...")
                 fetch_hashes(provider.repository, mapping)
@@ -116,7 +120,7 @@ def do_lock(
             raise
         except ResolutionImpossible as err:
             ui.echo(f"{termui.Emoji.LOCK} Lock failed", err=True)
-            ui.echo(format_resolution_impossible(err), err=True)
+            ui.error(format_resolution_impossible(err))
             raise ResolutionImpossible("Unable to find a resolution") from None
         else:
             data = format_lockfile(project, mapping, dependencies, groups=groups, strategy=lock_strategy)
@@ -128,7 +132,12 @@ def do_lock(
     return mapping
 
 
-def resolve_candidates_from_lockfile(project: Project, requirements: Iterable[Requirement]) -> dict[str, Candidate]:
+def resolve_candidates_from_lockfile(
+    project: Project,
+    requirements: Iterable[Requirement],
+    cross_platform: bool = False,
+    groups: Collection[str] | None = None,
+) -> dict[str, Candidate]:
     ui = project.core.ui
     resolve_max_rounds = int(project.config["strategy.resolve_max_rounds"])
     reqs = [
@@ -138,38 +147,51 @@ def resolve_candidates_from_lockfile(project: Project, requirements: Iterable[Re
         with ui.open_spinner("Resolving packages from lockfile...") as spinner:
             reporter = BaseReporter()
             provider = project.get_provider(for_install=True)
+            if cross_platform:
+                provider.repository.ignore_compatibility = True
+            if FLAG_INHERIT_METADATA in project.lockfile.strategy and groups is not None:
+                return {
+                    c.identify(): c for c in cast(LockedRepository, provider.repository).evaluate_candidates(groups)
+                }
             resolver: Resolver = project.core.resolver_class(provider, reporter)
-            mapping, *_ = resolve(
-                resolver,
-                reqs,
-                project.environment.python_requires,
-                resolve_max_rounds,
-            )
-            spinner.update("Fetching hashes for resolved packages...")
-            fetch_hashes(provider.repository, mapping)
-    return mapping
+            try:
+                mapping, *_ = resolve(
+                    resolver,
+                    reqs,
+                    project.environment.python_requires,
+                    resolve_max_rounds,
+                    inherit_metadata=cross_platform,
+                )
+            except ResolutionImpossible as e:
+                logger.exception("Broken lockfile")
+                raise PdmException(
+                    "Resolving from lockfile failed. You may fix the lockfile by `pdm lock --update-reuse` and retry."
+                ) from e
+            else:
+                spinner.update("Fetching hashes for resolved packages...")
+                fetch_hashes(provider.repository, mapping)
+                return mapping
 
 
 def check_lockfile(project: Project, raise_not_exist: bool = True) -> str | None:
-    """Check if the lock file exists and is up to date. Return the update strategy."""
+    """Check if the lock file exists and is up to date. Return the lock strategy."""
+    from pdm.project.lockfile import Compatibility
+
     if not project.lockfile.exists():
         if raise_not_exist:
-            raise ProjectError("Lock file does not exist, nothing to install")
-        project.core.ui.echo("Lock file does not exist", style="warning", err=True)
+            raise ProjectError("Lockfile does not exist, nothing to install")
+        project.core.ui.warn("Lockfile does not exist")
         return "all"
-    elif not project.lockfile.is_compatible():
-        project.core.ui.echo(
-            "Lock file version is not compatible with PDM, installation may fail",
-            style="warning",
-            err=True,
-        )
-        return "reuse"  # try to reuse the lockfile if possible
+    compat = project.lockfile.compatibility()
+    if compat == Compatibility.NONE:
+        project.core.ui.warn("Lockfile is not compatible with PDM")
+        return "reuse"
+    elif compat == Compatibility.BACKWARD:
+        project.core.ui.warn("Lockfile is generated on an older version of PDM")
+    elif compat == Compatibility.FORWARD:
+        project.core.ui.warn("Lockfile is generated on a newer version of PDM")
     elif not project.is_lockfile_hash_match():
-        project.core.ui.echo(
-            "Lock file hash doesn't match pyproject.toml, packages may be outdated",
-            style="warning",
-            err=True,
-        )
+        project.core.ui.warn("Lockfile hash doesn't match pyproject.toml, packages may be outdated")
         return "reuse"
     return None
 
@@ -196,7 +218,7 @@ def do_sync(
         selection.validate()
         for group in selection:
             requirements.extend(project.get_dependencies(group).values())
-    candidates = resolve_candidates_from_lockfile(project, requirements)
+    candidates = resolve_candidates_from_lockfile(project, requirements, groups=list(selection))
     if tracked_names and dry_run:
         candidates = {name: c for name, c in candidates.items() if name in tracked_names}
     synchronizer = project.core.synchronizer_class(
@@ -205,7 +227,7 @@ def do_sync(
         clean=clean,
         dry_run=dry_run,
         no_editable=no_editable,
-        install_self=not no_self and bool(project.name),
+        install_self=not no_self and project.is_distribution,
         reinstall=reinstall,
         only_keep=only_keep,
         fail_fast=fail_fast,
@@ -250,15 +272,8 @@ def print_pep582_command(project: Project, shell: str = "AUTO") -> None:
         try:
             set_env_in_reg("PYTHONPATH", pep582_path)
         except PermissionError:
-            ui.echo(
-                "Permission denied, please run the terminal as administrator.",
-                style="error",
-                err=True,
-            )
-        ui.echo(
-            "The environment variable has been saved, please restart the session to take effect.",
-            style="success",
-        )
+            ui.error("Permission denied, please run the terminal as administrator.")
+        ui.info("The environment variable has been saved, please restart the session to take effect.")
         return
     lib_path = pep582_path.replace("'", "\\'")
     if shell == "AUTO":
@@ -335,14 +350,6 @@ def check_update(project: Project) -> None:  # pragma: no cover
     if project.core.ui.verbosity < termui.Verbosity.NORMAL:
         return
 
-    if sys.version_info < (3, 8):
-        project.core.ui.echo(
-            "Python 3.7 has reached EOL in June 2023 and will be no longer supported starting from PDM 2.11.0. "
-            "Please upgrade to Python 3.8 or higher.",
-            err=True,
-            style="warning",
-        )
-
     this_version = project.core.version
     latest_version = get_latest_version(project)
     if latest_version is None or Version(this_version) >= Version(latest_version):
@@ -365,7 +372,7 @@ def check_update(project: Project) -> None:  # pragma: no cover
             install_command = f"{sys.executable} -m {install_command}"
 
     message = [
-        f"\nPDM [primary]{this_version}[/]",
+        f"PDM [primary]{this_version}[/]",
         f" is installed, while [primary]{latest_version}[/]",
         " is available.\n",
         f"Please run [req]`{install_command}`[/]",
@@ -373,60 +380,4 @@ def check_update(project: Project) -> None:  # pragma: no cover
         f"Run [req]`{disable_command}`[/]",
         " to disable the check.",
     ]
-    project.core.ui.echo("".join(message), err=True, style="info")
-
-
-# Moved functions
-def do_add(*args: Any, **kwargs: Any) -> None:  # pragma: no cover
-    from pdm.cli.commands.add import Command as AddCommand
-
-    deprecation_warning(
-        "`pdm.actions.do_add` has been moved to `pdm.cli.commands.add:Command.do_add` method, "
-        "This function will be removed in the future.",
-        stacklevel=2,
-    )
-    AddCommand().do_add(*args, **kwargs)
-
-
-def do_update(*args: Any, **kwargs: Any) -> None:  # pragma: no cover
-    from pdm.cli.commands.update import Command as UpdateCommand
-
-    deprecation_warning(
-        "`pdm.actions.do_update` has been moved to `pdm.cli.commands.update:Command.do_update` method, "
-        "This function will be removed in the future.",
-        stacklevel=2,
-    )
-    UpdateCommand().do_update(*args, **kwargs)
-
-
-def do_use(*args: Any, **kwargs: Any) -> None:  # pragma: no cover
-    from pdm.cli.commands.use import Command as UseCommand
-
-    deprecation_warning(
-        "`pdm.actions.do_use` has been moved to `pdm.cli.commands.use:Command.do_use` method, "
-        "This function will be removed in the future.",
-        stacklevel=2,
-    )
-    UseCommand().do_use(*args, **kwargs)
-
-
-def do_remove(*args: Any, **kwargs: Any) -> None:  # pragma: no cover
-    from pdm.cli.commands.remove import Command as RemoveCommand
-
-    deprecation_warning(
-        "`pdm.actions.do_remove` has been moved to `pdm.cli.commands.remove:Command.do_remove` method, "
-        "This function will be removed in the future.",
-        stacklevel=2,
-    )
-    RemoveCommand().do_remove(*args, **kwargs)
-
-
-def do_import(*args: Any, **kwargs: Any) -> None:  # pragma: no cover
-    from pdm.cli.commands.import_cmd import Command as ImportCommand
-
-    deprecation_warning(
-        "`pdm.actions.do_import` has been moved to `pdm.cli.commands.import_:Command.do_import` method, "
-        "This function will be removed in the future.",
-        stacklevel=2,
-    )
-    ImportCommand().do_import(*args, **kwargs)
+    project.core.ui.info("".join(message))
