@@ -9,6 +9,7 @@ import sys
 from collections import ChainMap, OrderedDict
 from concurrent.futures import ThreadPoolExecutor
 from fnmatch import fnmatch
+from gettext import gettext as _
 from json import dumps
 from pathlib import Path
 from typing import (
@@ -37,7 +38,7 @@ from pdm.models.requirements import (
     strip_extras,
 )
 from pdm.models.specifiers import PySpecSet, get_specifier
-from pdm.project.lockfile import FLAG_CROSS_PLATFORM, FLAG_STATIC_URLS
+from pdm.project.lockfile import FLAG_CROSS_PLATFORM, FLAG_INHERIT_METADATA, FLAG_STATIC_URLS
 from pdm.utils import (
     comparable_version,
     is_path_relative_to,
@@ -152,6 +153,13 @@ class ArgumentParser(argparse.ArgumentParser):
         )
         self._optionals.title = "options"
 
+    def parse_known_args(self, args: Any = None, namespace: Any = None) -> Any:
+        args, argv = super().parse_known_args(args, namespace)
+        if argv:
+            msg = _("unrecognized arguments: %s")
+            self.error(msg % " ".join(argv))
+        return args, argv
+
 
 class ErrorArgumentParser(ArgumentParser):
     """A subclass of argparse.ArgumentParser that raises
@@ -200,12 +208,7 @@ def build_dependency_graph(
         if dist:
             requirements = (
                 parse_requirement(r)
-                for r in filter_requirements_with_extras(
-                    cast(str, dist.metadata["Name"]),
-                    dist.requires or [],
-                    extras,
-                    include_default=True,
-                )
+                for r in filter_requirements_with_extras(dist.requires or [], extras, include_default=True)
             )
             for req in requirements:
                 if not req.marker or req.marker.evaluate(marker_env):
@@ -326,7 +329,7 @@ def add_package_to_reverse_tree(
 
 
 def package_is_project(package: Package, project: Project) -> bool:
-    return project.name is not None and package.name == normalize_name(project.name)
+    return project.is_distribution and package.name == normalize_name(project.name)
 
 
 def _format_forward_dependency_graph(
@@ -529,6 +532,10 @@ def format_lockfile(
         base = tomlkit.table()
         base.update(v.as_lockfile_entry(project.root))
         base.add("summary", v.summary or "")
+        if FLAG_INHERIT_METADATA in strategy:
+            base.add("groups", v.req.groups)
+            if v.req.marker is not None:
+                base.add("marker", str(v.req.marker))
         deps: list[str] = []
         for r in fetched_dependencies[v.dep_key]:
             # Try to convert to relative paths to make it portable
@@ -589,7 +596,7 @@ def save_version_specifiers(
 
     for reqs in requirements.values():
         for name, r in reqs.items():
-            if r.is_named and not r.specifier:
+            if r.is_named and not r.specifier and name in resolved:
                 if save_strategy == "exact":
                     r.specifier = get_specifier(f"=={candidate_version(resolved[name])}")
                 elif save_strategy == "compatible":
@@ -650,6 +657,8 @@ def format_resolution_impossible(err: ResolutionImpossible) -> str:
 
     causes: list[RequirementInformation] = err.causes
     info_lines: set[str] = set()
+    if not causes:
+        return ""
     if all(isinstance(cause.requirement, PythonRequirement) for cause in causes):
         project_requires: PythonRequirement = next(cause.requirement for cause in causes if cause.parent is None)
         pyspec = cast(PySpecSet, project_requires.specifier)
@@ -773,9 +782,7 @@ def use_venv(project: Project, name: str) -> None:
     from pdm.environments import PythonEnvironment
 
     venv = get_venv_with_name(project, cast(str, name))
-    project.core.ui.echo(
-        f"In virtual environment: [success]{venv.root}[/]", err=True, style="info", verbosity=termui.Verbosity.DETAIL
-    )
+    project.core.ui.info(f"In virtual environment: [success]{venv.root}[/]", verbosity=termui.Verbosity.DETAIL)
     project.environment = PythonEnvironment(project, python=str(venv.interpreter))
 
 
