@@ -1,16 +1,15 @@
 from __future__ import annotations
 
 import argparse
-import dataclasses
 import hashlib
 import shlex
 import urllib.parse
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Mapping
 
+from pdm.exceptions import PdmUsageError
 from pdm.formats.base import make_array
 from pdm.models.requirements import FileRequirement, Requirement, parse_requirement
-from pdm.models.specifiers import get_specifier
 from pdm.utils import expand_env_vars_in_auth
 
 if TYPE_CHECKING:
@@ -189,7 +188,7 @@ def export(
     collected_req: set[str] = set()
     for candidate in sorted(candidates, key=lambda x: x.identify()):  # type: ignore[attr-defined]
         if isinstance(candidate, Candidate):
-            req = dataclasses.replace(candidate.req, specifier=get_specifier(f"=={candidate.version}"), marker=None)
+            req = candidate.req.as_pinned_version(candidate.version)
         else:
             assert isinstance(candidate, Requirement)
             req = candidate
@@ -202,6 +201,13 @@ def export(
             for item in sorted({row["hash"] for row in candidate.hashes}):  # type: ignore[attr-defined]
                 lines.append(f" \\\n    --hash={item}")
         lines.append("\n")
+    if (options.self or options.editable_self) and not project.is_distribution:
+        raise PdmUsageError("Cannot export the project itself in a non-library project.")
+    if options.self:
+        lines.append(".  # this package\n")
+    elif options.editable_self:
+        lines.append("-e .  # this package\n")
+
     sources = project.pyproject.settings.get("source", [])
     for source in sources:
         url = source["url"]
