@@ -5,17 +5,17 @@ import functools
 import multiprocessing
 import traceback
 from concurrent.futures import Future, ThreadPoolExecutor
+from functools import cached_property
 from types import SimpleNamespace
 from typing import TYPE_CHECKING, Any, Callable, Collection, TypeVar
 
 from rich.progress import SpinnerColumn, TaskProgressColumn
 
 from pdm import termui
-from pdm.compat import cached_property
 from pdm.environments import BaseEnvironment
 from pdm.exceptions import InstallationError
 from pdm.installers.manager import InstallManager
-from pdm.models.candidates import Candidate, make_candidate
+from pdm.models.candidates import Candidate
 from pdm.models.reporter import BaseReporter, RichProgressReporter
 from pdm.models.requirements import FileRequirement, Requirement, parse_requirement, strip_extras
 from pdm.utils import is_editable, normalize_name
@@ -159,7 +159,7 @@ class BaseSynchronizer:
                 candidate = candidates[key]
                 # Create a new candidate with editable=False
                 req = dataclasses.replace(candidate.req, editable=False)
-                candidates[key] = make_candidate(req, candidate.name, candidate.version, candidate.link)
+                candidates[key] = candidate.copy_with(req)
         return candidates
 
     def should_install_editables(self) -> bool:
