@@ -1,12 +1,11 @@
 import sys
-import warnings
 
 from pdm import environments
+from pdm.utils import deprecation_warning
 
-warnings.warn(
+deprecation_warning(
     "pdm.models.environment is deprecated, please use pdm.environments instead. "
     "This module will be removed in the future.",
-    DeprecationWarning,
     stacklevel=1,
 )
 
