@@ -7,11 +7,11 @@ import re
 import sys
 import warnings
 from functools import wraps
-from typing import TYPE_CHECKING, Generator, TypeVar, cast
+from typing import TYPE_CHECKING, Collection, Generator, TypeVar, cast
 
 from pdm import termui
 from pdm.exceptions import CandidateInfoNotFound, CandidateNotFound, PackageWarning, PdmException
-from pdm.models.candidates import Candidate, make_candidate
+from pdm.models.candidates import Candidate
 from pdm.models.requirements import (
     Requirement,
     filter_requirements_with_extras,
@@ -21,6 +21,7 @@ from pdm.models.search import SearchResultParser
 from pdm.models.specifiers import PySpecSet
 from pdm.utils import (
     cd,
+    filtered_sources,
     normalize_name,
     path_to_url,
     url_to_path,
@@ -76,22 +77,7 @@ class BaseRepository:
 
     def get_filtered_sources(self, req: Requirement) -> list[RepositoryConfig]:
         """Get matching sources based on the index attribute."""
-        source_preferences = [(s, self.source_preference(req, s)) for s in self.sources]
-        included_by = [s for s, p in source_preferences if p is True]
-        if included_by:
-            return included_by
-        return [s for s, p in source_preferences if p is None]
-
-    @staticmethod
-    def source_preference(req: Requirement, source: RepositoryConfig) -> bool | None:
-        key = req.key
-        if key is None:
-            return None
-        if any(fnmatch.fnmatch(key, pat) for pat in source.include_packages):
-            return True
-        if any(fnmatch.fnmatch(key, pat) for pat in source.exclude_packages):
-            return False
-        return None
+        return filtered_sources(self.sources, req.key)
 
     def get_dependencies(self, candidate: Candidate) -> tuple[list[Requirement], PySpecSet, str]:
         """Get (dependencies, python_specifier, summary) of the candidate."""
@@ -138,7 +124,7 @@ class BaseRepository:
     def is_this_package(self, requirement: Requirement) -> bool:
         """Whether the requirement is the same as this package"""
         project = self.environment.project
-        return requirement.is_named and project.name is not None and requirement.key == normalize_name(project.name)
+        return requirement.is_named and project.is_distribution and requirement.key == normalize_name(project.name)
 
     def make_this_candidate(self, requirement: Requirement) -> Candidate:
         """Make a candidate for this package.
@@ -147,9 +133,8 @@ class BaseRepository:
         from unearth import Link
 
         project = self.environment.project
-        assert project.name
         link = Link.from_path(project.root)
-        candidate = make_candidate(requirement, project.name, link=link)
+        candidate = Candidate(requirement, project.name, link=link)
         candidate.prepare(self.environment).metadata
         return candidate
 
@@ -213,7 +198,10 @@ class BaseRepository:
                     warnings.warn(
                         f"Skipping {candidate.name}@{candidate.version} because it requires "
                         f"{python_specifier(candidate.requires_python)} but the project claims to work with "
-                        f"{python_specifier(project_requires_python)}.\nNarrow down the `requires-python` range to "
+                        f"{python_specifier(project_requires_python)}. Instead, another version of "
+                        f"{candidate.name} that supports {python_specifier(project_requires_python)} will "
+                        f"be used.\nIf you want to install {candidate.name}@{candidate.version}, "
+                        "narrow down the `requires-python` range to "
                         f'include this version. For example, "{working_requires_python}" should work.',
                         PackageWarning,
                         stacklevel=4,
@@ -287,7 +275,7 @@ class BaseRepository:
         """Adds the local package as a candidate only if the candidate
         name is the same as the local package."""
         project = self.environment.project
-        if not project.name or candidate.name != project.name:
+        if not project.is_distribution or candidate.name != project.name:
             raise CandidateInfoNotFound(candidate) from None
 
         reqs = project.pyproject.metadata.get("dependencies", [])
@@ -431,11 +419,7 @@ class PyPIRepository(BaseRepository):
                     requirement_lines = info["requires_dist"] or []
                 except KeyError:
                     requirement_lines = info["requires"] or []
-                requirements = filter_requirements_with_extras(
-                    cast(str, candidate.req.project_name),
-                    requirement_lines,
-                    candidate.req.extras or (),
-                )
+                requirements = filter_requirements_with_extras(requirement_lines, candidate.req.extras or ())
                 return requirements, requires_python, summary
         raise CandidateInfoNotFound(candidate)
 
@@ -474,12 +458,10 @@ class PyPIRepository(BaseRepository):
             session = finder.session
             resp = session.get(search_url, params={"q": query})
             if resp.status_code == 404:
-                self.environment.project.core.ui.echo(
+                self.environment.project.core.ui.warn(
                     f"{pypi_simple!r} doesn't support '/search' endpoint, fallback "
                     f"to {self.DEFAULT_INDEX_URL!r} now.\n"
                     "This may take longer depending on your network condition.",
-                    err=True,
-                    style="warning",
                 )
                 resp = session.get(f"{self.DEFAULT_INDEX_URL}/search", params={"q": query})
             parser = SearchResultParser()
@@ -521,7 +503,7 @@ class LockedRepository(BaseRepository):
                 req = Requirement.from_req_dict(package_name, req_dict)
                 if req.is_file_or_url and req.path and not req.url:  # type: ignore[attr-defined]
                     req.url = path_to_url(posixpath.join(root, req.path))  # type: ignore[attr-defined]
-                can = make_candidate(req, name=package_name, version=version)
+                can = Candidate(req, name=package_name, version=version)
                 can.hashes = package.get("files", [])
                 if not static_urls and any("url" in f for f in can.hashes):
                     raise PdmException(
@@ -554,7 +536,7 @@ class LockedRepository(BaseRepository):
     def _get_dependencies_from_lockfile(self, candidate: Candidate) -> CandidateInfo:
         err = (
             f"Missing package {candidate.identify()} from the lockfile, "
-            "the lockfile may be broken. Run `pdm update` to fix it."
+            "the lockfile may be broken. Run `pdm lock --update-reuse` to fix it."
         )
         try:
             return self.candidate_info[self._identify_candidate(candidate)]
@@ -612,3 +594,15 @@ class LockedRepository(BaseRepository):
 
     def get_hashes(self, candidate: Candidate) -> list[FileHash]:
         return candidate.hashes
+
+    def evaluate_candidates(self, groups: Collection[str]) -> Iterable[Candidate]:
+        for can in self.packages.values():
+            if not any(g in can.req.groups for g in groups):
+                continue
+            if (
+                not self.ignore_compatibility
+                and can.req.marker is not None
+                and not can.req.marker.evaluate(self.environment.marker_environment)
+            ):
+                continue
+            yield can
