@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import json
 import re
+import warnings
 from functools import lru_cache
 from operator import attrgetter
 from typing import Any, Iterable, Match, cast
@@ -19,7 +20,7 @@ def _read_max_versions() -> dict[Version, int]:
         return {Version(k): v for k, v in json.load(fp).items()}
 
 
-@lru_cache()
+@lru_cache
 def get_specifier(version_str: SpecifierSet | str) -> SpecifierSet:
     if isinstance(version_str, SpecifierSet):
         return version_str
@@ -31,26 +32,27 @@ def get_specifier(version_str: SpecifierSet | str) -> SpecifierSet:
 _legacy_specifier_re = re.compile(r"(==|!=|<=|>=|<|>)(\s*)([^,;\s)]*)")
 
 
-@lru_cache()
+@lru_cache
 def fix_legacy_specifier(specifier: str) -> str:
     """Since packaging 22.0, legacy specifiers like '>=4.*' are no longer
     supported. We try to normalize them to the new format.
     """
-    from pdm.utils import deprecation_warning
 
     def fix_wildcard(match: Match[str]) -> str:
         operator, _, version = match.groups()
         if operator in ("==", "!="):
             return match.group(0)
         if ".*" in version:
-            deprecation_warning(".* suffix can only be used with `==` or `!=` operators", stacklevel=4)
+            warnings.warn(".* suffix can only be used with `==` or `!=` operators", FutureWarning, stacklevel=4)
             version = version.replace(".*", ".0")
             if operator in ("<", "<="):  # <4.* and <=4.* are equivalent to <4.0
                 operator = "<"
             elif operator in (">", ">="):  # >4.* and >=4.* are equivalent to >=4.0
                 operator = ">="
         elif "+" in version:  # Drop the local version
-            deprecation_warning("Local version label can only be used with `==` or `!=` operators", stacklevel=4)
+            warnings.warn(
+                "Local version label can only be used with `==` or `!=` operators", FutureWarning, stacklevel=4
+            )
             version = version.split("+")[0]
         return f"{operator}{version}"
 
@@ -266,7 +268,7 @@ class PySpecSet(SpecifierSet):
         instance._excludes = self._excludes[:]
         return instance
 
-    @lru_cache()
+    @lru_cache
     def __and__(self, other: PySpecSet) -> PySpecSet:
         if any(s.is_impossible for s in (self, other)):
             return ImpossiblePySpecSet()
@@ -281,7 +283,7 @@ class PySpecSet(SpecifierSet):
         rv._rearrange(lower, upper, excludes)
         return rv
 
-    @lru_cache()
+    @lru_cache
     def __or__(self, other: PySpecSet) -> PySpecSet:
         if self.is_impossible:
             return other.copy()
@@ -340,7 +342,7 @@ class PySpecSet(SpecifierSet):
                     prev = prev.bump()
                 break
 
-    @lru_cache()
+    @lru_cache
     def is_superset(self, other: str | SpecifierSet) -> bool:
         if self.is_impossible:
             return False
@@ -356,7 +358,7 @@ class PySpecSet(SpecifierSet):
             return False
         return lower <= other._lower_bound and upper >= other._upper_bound and set(excludes) <= set(other._excludes)
 
-    @lru_cache()
+    @lru_cache
     def is_subset(self, other: str | SpecifierSet) -> bool:
         if self.is_impossible:
             return False
