@@ -6,6 +6,7 @@ import os
 import re
 import shutil
 import sys
+from functools import cached_property
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Callable, Iterable, Mapping, cast
 
@@ -14,7 +15,6 @@ from tomlkit.items import Array
 
 from pdm import termui
 from pdm._types import RepositoryConfig
-from pdm.compat import cached_property
 from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
 from pdm.models.backends import BuildBackend, get_backend_by_spec
 from pdm.models.python import PythonInfo
@@ -32,7 +32,6 @@ from pdm.utils import (
     find_python_in_path,
     is_conda_base,
     is_conda_base_python,
-    normalize_name,
     path_to_url,
 )
 
@@ -95,15 +94,14 @@ class Project:
             root_path = global_project
             is_global = True
             if self.global_config["global_project.fallback_verbose"]:
-                self.core.ui.echo(
-                    "Project is not found, fallback to the global project",
-                    style="warning",
-                    err=True,
-                )
+                self.core.ui.info("Project is not found, fallback to the global project")
 
         self.root: Path = Path(root_path or "").absolute()
         self.is_global = is_global
-        self.enable_write_lockfile = os.getenv("PDM_NO_LOCK", "0").lower() not in ("1", "true")
+        self.enable_write_lockfile = os.getenv("PDM_FROZEN_LOCKFILE", os.getenv("PDM_NO_LOCK", "0")).lower() not in (
+            "1",
+            "true",
+        )
         self.init_global_project()
 
     def __repr__(self) -> str:
@@ -144,7 +142,7 @@ class Project:
         return config
 
     @property
-    def name(self) -> str | None:
+    def name(self) -> str:
         return self.pyproject.metadata.get("name")
 
     @property
@@ -200,7 +198,7 @@ class Project:
 
         def note(message: str) -> None:
             if not self.is_global:
-                self.core.ui.echo(message, style="info", err=True)
+                self.core.ui.info(message)
 
         config = self.config
         saved_path = self._saved_python
@@ -306,11 +304,9 @@ class Project:
             deps = metadata.get("dependencies", [])
         else:
             if group in optional_dependencies and group in dev_dependencies:
-                self.core.ui.echo(
-                    f"The {group} group exists in both [optional-dependencies] "
-                    "and [dev-dependencies], the former is taken.",
-                    err=True,
-                    style="warning",
+                self.core.ui.info(
+                    f"The {group} group exists in both \\[optional-dependencies] "
+                    "and \\[dev-dependencies], the former is taken."
                 )
             if group in optional_dependencies:
                 deps = optional_dependencies[group]
@@ -323,17 +319,16 @@ class Project:
             for line in deps:
                 if line.startswith("-e "):
                     if in_metadata:
-                        self.core.ui.echo(
-                            f"WARNING: Skipping editable dependency [b]{line}[/] in the"
+                        self.core.ui.warn(
+                            f"Skipping editable dependency [b]{line}[/] in the"
                             r" [success]\[project][/] table. Please move it to the "
-                            r"[success]\[tool.pdm.dev-dependencies][/] table",
-                            err=True,
-                            style="warning",
+                            r"[success]\[tool.pdm.dev-dependencies][/] table"
                         )
                         continue
                     req = parse_requirement(line[3:].strip(), True)
                 else:
                     req = parse_requirement(line)
+                req.groups = [group]
                 # make editable packages behind normal ones to override correctly.
                 result[req.identify()] = req
         return result
@@ -350,10 +345,6 @@ class Project:
     def all_dependencies(self) -> dict[str, dict[str, Requirement]]:
         return {group: self.get_dependencies(group) for group in self.iter_groups()}
 
-    @property
-    def allow_prereleases(self) -> bool | None:
-        return self.pyproject.settings.get("allow_prereleases")
-
     @property
     def default_source(self) -> RepositoryConfig:
         """Get the default source from the pypi setting"""
@@ -431,46 +422,42 @@ class Project:
         :returns: The provider object
         """
 
-        from pdm.resolver.providers import (
-            BaseProvider,
-            EagerUpdateProvider,
-            ReusePinProvider,
-        )
+        from pdm.resolver.providers import BaseProvider, get_provider, provider_arguments
 
         repository = self.get_repository(ignore_compatibility=ignore_compatibility)
-        allow_prereleases = self.allow_prereleases
-        overrides = {normalize_name(k): v for k, v in self.pyproject.resolution_overrides.items()}
         locked_repository: LockedRepository | None = None
-        if strategy != "all" or for_install:
-            try:
-                locked_repository = self.locked_repository
-            except Exception:
-                if for_install:
-                    raise
-                self.core.ui.echo(
-                    "Unable to reuse the lock file as it is not compatible with PDM",
-                    style="warning",
-                    err=True,
-                )
+        try:
+            locked_repository = self.locked_repository
+        except Exception:  # pragma: no cover
+            if for_install:
+                raise
+            if strategy != "all":
+                self.core.ui.warn("Unable to reuse the lock file as it is not compatible with PDM")
 
-        if locked_repository is None:
-            return BaseProvider(
-                repository, allow_prereleases, overrides, direct_minimal_versions=direct_minimal_versions
-            )
         if for_install:
+            assert locked_repository is not None
             return BaseProvider(
-                locked_repository, allow_prereleases, overrides, direct_minimal_versions=direct_minimal_versions
+                locked_repository, direct_minimal_versions=direct_minimal_versions, locked_candidates={}
             )
-        provider_class = ReusePinProvider if strategy == "reuse" else EagerUpdateProvider
-        tracked_names = [strip_extras(name)[0] for name in tracked_names or ()]
-        return provider_class(
-            locked_repository.all_candidates,
-            tracked_names,
-            repository,
-            allow_prereleases,
-            overrides,
-            direct_minimal_versions=direct_minimal_versions,
-        )
+        provider_class = get_provider(strategy)
+        params: dict[str, Any] = {}
+        if strategy != "all":
+            params["tracked_names"] = [strip_extras(name)[0] for name in tracked_names or ()]
+        locked_candidates = {} if locked_repository is None else locked_repository.all_candidates
+        accepted_args = provider_arguments(provider_class)
+        if "locked_candidates" in accepted_args:
+            params["locked_candidates"] = locked_candidates
+        elif "preferred_pins" in accepted_args:  # pragma: no cover
+            deprecation_warning(
+                "`preferred_pins` has been moved to keyword-only argument `locked_candidates`", stacklevel=1
+            )
+            params["preferred_pins"] = locked_candidates
+        else:  # pragma: no cover
+            deprecation_warning(
+                "Missing `locked_candidates` argument from the provider class, it will be populated automatically",
+                stacklevel=1,
+            )
+        return provider_class(repository=repository, direct_minimal_versions=direct_minimal_versions, **params)
 
     def get_reporter(
         self,
@@ -494,7 +481,7 @@ class Project:
 
     def get_lock_metadata(self) -> dict[str, Any]:
         content_hash = "sha256:" + self.pyproject.content_hash("sha256")
-        return {"lock_version": self.lockfile.spec_version, "content_hash": content_hash}
+        return {"lock_version": str(self.lockfile.spec_version), "content_hash": content_hash}
 
     def write_lockfile(self, toml_data: dict, show_message: bool = True, write: bool = True, **_kwds: Any) -> None:
         """Write the lock file to disk."""
@@ -509,12 +496,12 @@ class Project:
     def make_self_candidate(self, editable: bool = True) -> Candidate:
         from unearth import Link
 
-        from pdm.models.candidates import make_candidate
+        from pdm.models.candidates import Candidate
 
         req = parse_requirement(path_to_url(self.root.as_posix()), editable)
         assert self.name
         req.name = self.name
-        can = make_candidate(req, name=self.name, link=Link.from_path(self.root))
+        can = Candidate(req, name=self.name, link=Link.from_path(self.root))
         can.prepare(self.environment).metadata
         return can
 
@@ -681,13 +668,14 @@ class Project:
             finder.add_provider(VenvProvider(self), venv_pos)
         return finder
 
-    # compatibility, shouldn't be used directly
     @property
-    def meta(self) -> dict[str, Any]:
-        deprecation_warning("project.meta is deprecated, use project.pyproject.metadata instead", stacklevel=2)
-        return self.pyproject.metadata
-
-    @property
-    def tool_settings(self) -> dict[str, Any]:
-        deprecation_warning("project.tool_settings is deprecated, use project.pyproject.settings instead", stacklevel=2)
-        return self.pyproject.settings
+    def is_distribution(self) -> bool:
+        if not self.name:
+            return False
+        settings = self.pyproject.settings
+        if "package-type" in settings:
+            return settings["package-type"] == "library"
+        elif "distribution" in settings:
+            return settings["distribution"]
+        else:
+            return True
