@@ -1,24 +1,21 @@
 from __future__ import annotations
 
+import inspect
 import os
 from typing import TYPE_CHECKING, Callable, cast
 
 from packaging.specifiers import InvalidSpecifier, SpecifierSet
+from packaging.version import InvalidVersion, Version
 from resolvelib import AbstractProvider, RequirementsConflicted
 from resolvelib.resolvers import Criterion
 
 from pdm.exceptions import InvalidPyVersion, RequirementError
-from pdm.models.candidates import Candidate, make_candidate
+from pdm.models.candidates import Candidate
 from pdm.models.repositories import LockedRepository
 from pdm.models.requirements import FileRequirement, parse_requirement, strip_extras
-from pdm.resolver.python import (
-    PythonCandidate,
-    PythonRequirement,
-    find_python_matches,
-    is_python_satisfied_by,
-)
+from pdm.resolver.python import PythonCandidate, PythonRequirement, find_python_matches, is_python_satisfied_by
 from pdm.termui import logger
-from pdm.utils import is_url, url_without_fragments
+from pdm.utils import deprecation_warning, is_url, normalize_name, url_without_fragments
 
 if TYPE_CHECKING:
     from typing import Any, Iterable, Iterator, Mapping, Sequence
@@ -30,6 +27,34 @@ if TYPE_CHECKING:
     from pdm.models.requirements import Requirement
 
 
+_PROVIDER_REGISTORY: dict[str, type[BaseProvider]] = {}
+
+
+def get_provider(strategy: str) -> type[BaseProvider]:
+    return _PROVIDER_REGISTORY[strategy]
+
+
+def provider_arguments(provider: type[BaseProvider]) -> set[str]:
+    arguments: set[str] = set()
+    for cls in provider.__mro__:
+        if "__init__" not in cls.__dict__:
+            continue
+        params = inspect.signature(cls).parameters
+        arguments.update({k for k, v in params.items() if v.kind not in (v.VAR_POSITIONAL, v.VAR_KEYWORD)})
+        if not any(p.kind in (p.VAR_POSITIONAL, p.VAR_KEYWORD) for p in params.values()):
+            break
+    return arguments
+
+
+def register_provider(strategy: str) -> Callable[[type[BaseProvider]], type[BaseProvider]]:
+    def wrapper(cls: type[BaseProvider]) -> type[BaseProvider]:
+        _PROVIDER_REGISTORY[strategy] = cls
+        return cls
+
+    return wrapper
+
+
+@register_provider("all")
 class BaseProvider(AbstractProvider):
     def __init__(
         self,
@@ -37,12 +62,33 @@ class BaseProvider(AbstractProvider):
         allow_prereleases: bool | None = None,
         overrides: dict[str, str] | None = None,
         direct_minimal_versions: bool = False,
+        locked_candidates: dict[str, Candidate] | None = None,
     ) -> None:
+        if overrides is not None:  # pragma: no cover
+            deprecation_warning(
+                "The `overrides` argument is deprecated and will be removed in the future.", stacklevel=2
+            )
+        if allow_prereleases is not None:  # pragma: no cover
+            deprecation_warning(
+                "The `allow_prereleases` argument is deprecated and will be removed in the future.", stacklevel=2
+            )
+        project = repository.environment.project
+        if locked_candidates is None:  # pragma: no cover
+            try:
+                locked_repository = project.locked_repository
+            except Exception:
+                locked_candidates = {}
+            else:
+                locked_candidates = locked_repository.all_candidates
         self.repository = repository
-        self.allow_prereleases = allow_prereleases  # Root allow_prereleases value
+        self.allow_prereleases = project.pyproject.allow_prereleases  # Root allow_prereleases value
         self.fetched_dependencies: dict[tuple[str, str | None], list[Requirement]] = {}
-        self.overrides = overrides or {}
+        self.overrides: Mapping[str, str] = {
+            normalize_name(k): v for k, v in project.pyproject.resolution.get("overrides", {}).items()
+        }
+        self.excludes = {normalize_name(k) for k in project.pyproject.resolution.get("excludes", [])}
         self.direct_minimal_versions = direct_minimal_versions
+        self.locked_candidates = locked_candidates
         self._known_depth: dict[str, int] = {}
 
     def requirement_preference(self, requirement: Requirement) -> Comparable:
@@ -56,7 +102,9 @@ class BaseProvider(AbstractProvider):
         is_named = requirement.is_named
         is_pinned = requirement.is_pinned
         is_prerelease = (
-            requirement.prerelease or requirement.specifier is not None and bool(requirement.specifier.prereleases)
+            bool(requirement.prerelease)
+            or requirement.specifier is not None
+            and bool(requirement.specifier.prereleases)
         )
         specifier_parts = len(requirement.specifier) if requirement.specifier else 0
         return (not editable, is_named, not is_pinned, not is_prerelease, -specifier_parts)
@@ -134,14 +182,26 @@ class BaseProvider(AbstractProvider):
 
     def _find_candidates(self, requirement: Requirement) -> Iterable[Candidate]:
         if not requirement.is_named and not isinstance(self.repository, LockedRepository):
-            can = make_candidate(requirement)
+            can = Candidate(requirement)
             if not can.name:
                 can.prepare(self.repository.environment).metadata
             return [can]
         else:
+            prerelease = requirement.prerelease
+            if prerelease is None and (key := requirement.identify()) in self.locked_candidates:
+                # keep the prerelease if it is locked
+                candidate = self.locked_candidates[key]
+                if candidate.version is not None:
+                    try:
+                        parsed_version = Version(candidate.version)
+                    except InvalidVersion:  # pragma: no cover
+                        pass
+                    else:
+                        if parsed_version.is_prerelease:
+                            prerelease = True
             return self.repository.find_candidates(
                 requirement,
-                requirement.prerelease or self.allow_prereleases,
+                self.allow_prereleases if prerelease is None else prerelease,
                 minimal_version=self.direct_minimal_versions and self._is_direct_requirement(requirement),
             )
 
@@ -159,6 +219,8 @@ class BaseProvider(AbstractProvider):
             elif identifier in self.overrides:
                 return iter(self.get_override_candidates(identifier))
             reqs = sorted(requirements[identifier], key=self.requirement_preference)
+            if not reqs:
+                return iter(())
             original_req = reqs[0]
             bare_name, extras = strip_extras(identifier)
             if extras and bare_name in requirements:
@@ -229,6 +291,8 @@ class BaseProvider(AbstractProvider):
                 & self.repository.environment.python_requires
             ).is_impossible:
                 continue
+            if dep.identify() in self.excludes:
+                continue
             dep.requires_python &= candidate.req.requires_python
             valid_deps.append(dep)
         self.fetched_dependencies[candidate.dep_key] = valid_deps[:]
@@ -248,6 +312,7 @@ class BaseProvider(AbstractProvider):
         return valid_deps
 
 
+@register_provider("reuse")
 class ReusePinProvider(BaseProvider):
     """A provider that reuses preferred pins if possible.
 
@@ -255,17 +320,21 @@ class ReusePinProvider(BaseProvider):
     where already-pinned candidates in lockfile should be preferred.
     """
 
-    def __init__(
-        self,
-        preferred_pins: dict[str, Candidate],
-        tracked_names: Iterable[str],
-        *args: Any,
-        **kwargs: Any,
-    ) -> None:
+    def __init__(self, *args: Any, tracked_names: Iterable[str], **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
-        self.preferred_pins = preferred_pins
         self.tracked_names = set(tracked_names)
 
+    def get_reuse_candidate(self, identifier: str, requirement: Requirement | None) -> Candidate | None:
+        bare_name = strip_extras(identifier)[0]
+        if bare_name in self.tracked_names:
+            return None
+        pin = self.locked_candidates.get(identifier)
+        if pin is None:
+            return None
+        if requirement is not None:
+            pin.req = requirement
+        return pin
+
     def find_matches(
         self,
         identifier: str,
@@ -273,15 +342,12 @@ class ReusePinProvider(BaseProvider):
         incompatibilities: Mapping[str, Iterator[Candidate]],
     ) -> Callable[[], Iterator[Candidate]]:
         super_find = super().find_matches(identifier, requirements, incompatibilities)
-        bare_name = strip_extras(identifier)[0]
 
         def matches_gen() -> Iterator[Candidate]:
-            if bare_name not in self.tracked_names and identifier in self.preferred_pins:
-                pin = self.preferred_pins[identifier]
+            requested_req = next(filter(lambda r: r.is_named, requirements[identifier]), None)
+            pin = self.get_reuse_candidate(identifier, requested_req)
+            if pin is not None:
                 incompat = list(incompatibilities[identifier])
-                demanded_req = next(requirements[identifier], None)
-                if demanded_req and demanded_req.is_named:
-                    pin.req = demanded_req
                 pin._preferred = True  # type: ignore[attr-defined]
                 if pin not in incompat and all(self.is_satisfied_by(r, pin) for r in requirements[identifier]):
                     yield pin
@@ -290,6 +356,7 @@ class ReusePinProvider(BaseProvider):
         return matches_gen
 
 
+@register_provider("eager")
 class EagerUpdateProvider(ReusePinProvider):
     """A specialized provider to handle an "eager" upgrade strategy.
 
@@ -302,12 +369,11 @@ class EagerUpdateProvider(ReusePinProvider):
     specified packages to upgrade, and free their pins when it has a chance.
     """
 
-    def is_satisfied_by(self, requirement: Requirement, candidate: Candidate) -> bool:
-        # If this is a tracking package, tell the resolver out of using the
-        # preferred pin, and into a "normal" candidate selection process.
-        if requirement.key in self.tracked_names and getattr(candidate, "_preferred", False):
-            return False
-        return super().is_satisfied_by(requirement, candidate)
+    def get_reuse_candidate(self, identifier: str, requirement: Requirement | None) -> Candidate | None:
+        if identifier in self.tracked_names:
+            # If this is a tracked package, don't reuse its pinned version, so it can be upgraded.
+            return None
+        return super().get_reuse_candidate(identifier, requirement)
 
     def get_dependencies(self, candidate: Candidate) -> list[Requirement]:
         # If this package is being tracked for upgrade, remove pins of its
@@ -330,3 +396,19 @@ class EagerUpdateProvider(ReusePinProvider):
         # Resolve tracking packages so we have a chance to unpin them first.
         (python, *others) = super().get_preference(identifier, resolutions, candidates, information, backtrack_causes)
         return (python, identifier not in self.tracked_names, *others)
+
+
+@register_provider("reuse-installed")
+class ReuseInstalledProvider(ReusePinProvider):
+    """A provider that reuses installed packages if possible."""
+
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
+        self.installed = self.repository.environment.get_working_set()
+
+    def get_reuse_candidate(self, identifier: str, requirement: Requirement | None) -> Candidate | None:
+        key = strip_extras(identifier)[0]
+        if key not in self.installed or requirement is None:
+            return super().get_reuse_candidate(identifier, requirement)
+        dist = self.installed[key]
+        return Candidate(requirement, name=dist.metadata["Name"], version=dist.metadata["Version"])
